function [IP,pf,A,pb,B,ef,eb,vaic,Vaicv]=mvar(u,maxIP,alg,criterion)
% Estimate MVAR
%
%[IP,pf,A,pb,B,ef,eb,vaic,Vaicv] = mvar(u,maxIP,alg,criterion)
%
% input: u     - data rows
%        maxIP - externaly defined maximum IP
%        alg   - for algorithm (=1 Nutall-Strand), (=2 - mlsm) ,
%                              (=3 - Vieira Morf), (=4 - QR artfit)
%        criterion for order choice - 1: AIC; 2: Hanna-Quinn, 3 Schwartz,
%                                     4 FPE, 5 - fixed order in maxIP
%                                     6 - xdelay,  7,8 - Broersen
%                                     9 - derivative (future) (envelope)
%                                     10 - estimate up to max order
%                                     Negative - keep criterion changes
% output: See Data Structure Data Base
%

stopFlag=0;
[nSegLength,nChannels] = size(u');
if nargin<3 alg=1; criterion=1; end % defaults
if nargin<4 criterion=1; end        % defaults
if criterion<0, stopFlag=1; criterion=abs(criterion); end

if criterion==5
   if maxIP==0
      pf=u*u';     % Eq. (15.90) Equation numbering refers to Marple Jr.
      pb=pf;       % Eq. (15.90)
      ef=u;
      eb=u;
      npf=size(pf);
      A=zeros(npf,npf,0);
      B=A;
      vaic=det(pf);
      Vaicv=det(pf);
      IP=0;
      return
   end
   IP=maxIP;
   if  alg==1,
      %  Formula from Marple Jr.
      [pf A pb B ef eb]=mcarns(u,IP);
   elseif alg==2,
      [pf A ef]=cmlsm(u,IP);
      B  = [ ]; eb = [ ]; pb = [ ];
   elseif alg==3,
      [pf A pb B ef eb]=mcarvm(u,IP);
   elseif alg==4,
      [pf A ef]=arfitcaps(u,IP);
      B  = [ ]; eb = [ ]; pb = [ ];
   end

   vaic=length(u)*log(det(pf))+2*(nChannels*nChannels)*IP;

   Vaicv=vaic;
   fprintf('IP=%0.f  vaic=%f\n',IP,vaic);
   
   if alg==1 | alg==3, %Nuttall-Strand and Vieira-Morf need scaling.
      pf=pf(:,:)/nSegLength;
   elseif alg==4,
      pf=pf; % Arfit does not need scaling.
   else %
      pf=pf(:,:)/nSegLength;
   end;

   return
end
if criterion==6
   IP=xdelay(u', maxIP);
   if  alg==1,
      %  Formula from Marple Jr.
      [pf A pb B ef eb]=mcarns(u,IP);
   elseif alg==2
      [pf A ef]=cmlsm(u,IP);
   elseif alg==3
      [pf A pb B ef eb]=mcarvm(u,IP);
   elseif alg==4
      [pf A ef]=arfitcaps(u,IP);
      %
   end
   vaic=0;
   Vaicv=0;

   if alg==1 | alg==3, %Nuttall-Strand and Vieira-Morf need scaling.
      pf=pf(:,:)/nSegLength;
   elseif alg==4,
      pf=pf; % Arfit does not need scaling.
   else %
      pf=pf(:,:)/nSegLength;
   end;

   return
end
%
vaicv=0;
if nargin<2
   maxOrder=30;
   disp(['maxOrder limited to ' int2str(maxOrder)])
   UpperboundOrder = round(3*sqrt(nSegLength)/nChannels); % Marple Jr. p. 409
   % Suggested by Nuttall, 1976.
   UpperboundOrder = min([maxOrder UpperboundOrder]);
else
   maxOrder=maxIP;
   UpperboundOrder=maxIP;
   disp(['maxOrder limited to ' int2str(maxOrder)])
end
IP=1;
Vaicv=zeros(maxOrder+1,1);
while IP <= UpperboundOrder,
   if  alg==1,
      %  Formula from Marple Jr.
      [npf na npb nb nef neb]=mcarns(u,IP);
   elseif alg==2
      [npf na nef]=cmlsm(u,IP);
   elseif alg==3
      [npf na npb nb nef neb]=mcarvm(u,IP);
   elseif alg==4
      [npf na nef]=arfitcaps(u,IP);
   end
   %  criterion
   if     criterion==1, % AIC
      vaic=length(u)*log(det(npf))+2*(nChannels*nChannels)*IP;
   elseif criterion==2  % Hannan-Quinn
      vaic=length(u)*log(det(npf)) ...
         + 2*log(log(length(u)))*(nChannels*nChannels)*IP;
   elseif criterion==3 % Schwartz
      vaic=length(u)*log(det(npf))+(log(length(u)))*(nChannels*nChannels)*IP;
   elseif criterion==4 % FPE
      vaic=log(det(npf)*((length(u)+nChannels*IP+1) ...
         /length(u)-nChannels*IP-1)^nChannels);
   elseif criterion==8 % Broersen
      vaic=log(det(npf))+2*(IP*nChannels^2+1)/(length(u)-(IP*nChannels^2+2));
   elseif criterion==7 % Broersen
      ml=1:IP;
      vaic=prod((length(u) ...
         + 1-nChannels*(ml-1))./(length(u)+1-nChannels*(ml+1)))*abs(det(npf));
      % care with maxiumum IP because vaic must be nonnegative
   end
   Vaicv(IP+1)=vaic;
   fprintf('IP=%0.f  vaic=%f\n',IP,vaic);
   if (vaic>vaicv) & (IP~=1),
      vaic=vaicv;
      if ~stopFlag, break; end;
   end; % Akaike condition
   vaicv=vaic;
   pf = npf;
   A  = na;
   ef = nef;
   if alg==1
      B  = nb;
      eb = neb;
      pb = npb;
   else % review status for backward prediction in clmsm
      B  = [ ];
      eb = [ ];
      pb = [ ];
   end
   IP=IP+1;
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Other criterions - D.Y.T.+K.S.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp(' ')
IP=IP-1;
vaic=vaicv;
Vaicv=Vaicv(2:IP+1);

if alg==1 | alg==3, %Nuttall-Strand and Vieira-Morf need scaling.
   pf=pf(:,:)/nSegLength;
elseif alg==4,
   pf=pf; % Arfit does not need scaling.
else %
   pf=pf(:,:)/nSegLength;
end;
